/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  ** This notice applies to any and all portions of this file
  * that are not between comment pairs USER CODE BEGIN and
  * USER CODE END. Other portions of this file, whether 
  * inserted by the user or by software development tools
  * are owned by their respective copyright owners.
  *
  * COPYRIGHT(c) 2018 STMicroelectronics
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */
/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "stm32l0xx_hal.h"
#include "dma.h"
#include "i2c.h"
#include "rtc.h"
#include "usart.h"
#include "gpio.h"

/* USER CODE BEGIN Includes */

#include "memory.h"
#include "charger.h"
#include "uartRxbuff.h"
#include "l80.h"
#include "BLE.h"
#include "murata.h"
#include "led.h"

/* USER CODE END Includes */

/* Private variables ---------------------------------------------------------*/

/* USER CODE BEGIN PV */
/* Private variables ---------------------------------------------------------*/
MainDevice mDevice;
__IO uint8_t acc_int = 0;
__IO uint8_t rect_int = 0;
/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
static void MX_NVIC_Init(void);

/* USER CODE BEGIN PFP */
/* Private function prototypes -----------------------------------------------*/

/* USER CODE END PFP */

/* USER CODE BEGIN 0 */

void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin)
{
	if ( GPIO_Pin == ACC_INT_Pin ) {
		if ( acc_int < 255 ) {
			acc_int++;
		}
	}else if ( GPIO_Pin == RECT_Pin ) {
		if ( rect_int < 255 ) {
			rect_int++;
		}
	}
	//send_position = 1;
}
__IO uint8_t alarmAtimeout = 0;
__IO uint8_t alarmBtimeout = 0;

uint8_t isRTCTimeout( uint8_t timeout ) {
	uint8_t val = 0;
	if ( timeout == 0 ) {
		val = alarmAtimeout;
		alarmAtimeout = 0;
	}else{
		val = alarmBtimeout;
		alarmBtimeout = 0;
	}
	return val;
}

void HAL_RTC_AlarmAEventCallback(RTC_HandleTypeDef *hrtc)
{
  /* Prevent unused argument(s) compilation warning */
  //if ( hrtc->Instance-> )
	asm( "nop" );
  alarmAtimeout = 1;
}


void storageModeStep() {
	while( 1 ) {
		if ( isChargerActive() == DEVICE_CHARGING ) {
			blinkLed( LED2, 1000 );
			HAL_Delay( 1000 );
			blinkLed( LED2, 1000 );
			break;
		}
		//HAL_PWR_EnterSTANDBYMode();
		HAL_PWR_EnterSTOPMode( PWR_LOWPOWERREGULATOR_ON, PWR_STOPENTRY_WFI );
	}

	radio_send_system_msg( 0,1,0 );

	mDevice.mode = NEAR_BEACON;
	writeDeviceStateToFlash( mDevice.mode );
}

void setUpTimeoutAlarm ( uint16_t timeout ) {

      RTC_TimeTypeDef sTime;
      RTC_DateTypeDef sDate;
      RTC_AlarmTypeDef sAlarm;

	  sTime.Hours = 0x0;
	  sTime.Minutes = 0x0;
	  sTime.Seconds = 0x0;
	  sTime.DayLightSaving = RTC_DAYLIGHTSAVING_NONE;
	  sTime.StoreOperation = RTC_STOREOPERATION_RESET;
	  if (HAL_RTC_SetTime(&hrtc, &sTime, RTC_FORMAT_BCD) != HAL_OK)
	  {
	  }

	  sDate.WeekDay = RTC_WEEKDAY_MONDAY;
	  sDate.Month = RTC_MONTH_JANUARY;
	  sDate.Date = 0x1;
	  sDate.Year = 0x0;
	  if (HAL_RTC_SetDate(&hrtc, &sDate, RTC_FORMAT_BCD) != HAL_OK)
	  {
	  }

		//HAL_RTCEx_BKUPWrite(&hrtc,RTC_BKP_DR0,0x32F2);
  //}
    /**Enable the Alarm A
    */

	  uint16_t alarm = 0;

	  sAlarm.AlarmTime.Hours = 0x0;
	  sAlarm.AlarmTime.Minutes = timeout / 60;
	  sAlarm.AlarmTime.Seconds = timeout % 60;
	  sAlarm.AlarmTime.SubSeconds = 0x0;
	  sAlarm.AlarmTime.DayLightSaving = RTC_DAYLIGHTSAVING_NONE;
	  sAlarm.AlarmTime.StoreOperation = RTC_STOREOPERATION_RESET;
	  sAlarm.AlarmMask = RTC_ALARMMASK_NONE;
	  sAlarm.AlarmSubSecondMask = RTC_ALARMSUBSECONDMASK_ALL;
	  sAlarm.AlarmDateWeekDaySel = RTC_ALARMDATEWEEKDAYSEL_DATE;
	  sAlarm.AlarmDateWeekDay = 0x1;
	  sAlarm.Alarm = RTC_ALARM_A;
	  if (HAL_RTC_SetAlarm_IT(&hrtc, &sAlarm, RTC_FORMAT_BCD) != HAL_OK)
	  {
	  }

}



void getAlarmStatus() {

	radio_send_KA( 20 );

}

void awayFromBeaconModeStep() {
	uint8_t alarm_received = 0;
	LIS3MDLinit();
	while( 1 ) {


		if ( getBeaconStatus() == BEACON_IN_RANGE ) {
			mDevice.mode = NEAR_BEACON;
			blinkLed( LED2, 2000 );
			HAL_Delay( 1000 );
			blinkLed( LED2, 2000 );
			break;
		}

		getAlarmStatus();

		if ( ALARM_STATUS == ALARM ) {
			mDevice.mode = ALARM_AND_AWAY;
			break;
		}



		setUpTimeoutAlarm( DEFAULT_NEAR_BEACON_CHECK_TIMEOUT );
		HAL_PWR_EnterSTOPMode( PWR_LOWPOWERREGULATOR_ON, PWR_STOPENTRY_WFI );

	}


}

void encodeGPSPoint( char * latitude, char * longitude, GPSPoint * point ) {
	uint8_t pos = 0;

	for( int x = 0; x < 10; x++ ) {
		if ( latitude[ x ] == '.' ) {
			pos++;
		}
		latitude[ x ] = latitude[ pos ];
		pos++;
	}

	latitude[ 9 ] = '\0';
	point->lattitude = atoi( latitude );
	point->lattitude = point->lattitude >> 8;

	pos = 0;

	for( int x = 0; x < 11; x++ ) {
		if ( longitude[ x ] == '.' ) {
			pos++;
		}
		longitude[ x ] = longitude[ pos ];
		pos++;
	}

	longitude[ 10 ] = '\0';
	point->longitude = atoi( longitude );
	point->longitude = point->longitude >> 8;
}


void sendGPSCoordinates( GPSPoint * point ) {
	//findMatchAndValueBlocking()

	uint8_t ss = findMatchWild ( &huart1buffer, GPGGA, strlen( GPGGA ),100 );
	if ( ss == -1 )
		ss = findMatchWild ( &huart1buffer, GPGGA, strlen( GPGGA ),100 );

	if ( ss == -1 )
		return -1;
	huart1buffer.data[ 99 ] = '\0';


	/*
	while( 1 ) {
		HAL_UART_Transmit( &huart2, "WAAAAAAAAAA", 8, 100 );
	}
	*/
	char output[ 200 ];

	ss = copyFromBuffer( &huart1buffer, strlen( "$GPGGA,**********,*********,*,*********,*,2" ), ss, &output );

	GPSPoint current_pos;

	if ( ss == -1 )
		return -1;

	char latitude[ 15 ];
	memcpy( latitude, output + 18, 9 );
	char longitude[ 15 ];
	memcpy( longitude, output + 30, 9 );



	encodeGPSPoint( latitude, longitude, point );



}

void sendGPS( int32_t lat, int32_t lon ) {
	char * gg;
	char buffer[ 140 ];
	sprintf( buffer, "%08X%08X", lat, lon );

	radio_on();
	radio_send_multibyte_msg( buffer, strlen( buffer ) );
	radio_off();
}

void sendEncodedGPS( uint8_t * lat, uint8_t * lon, uint64_t time ) {
	  char buffer[ 90 ];
	  int32_t lattitude = atoi( lat );
	  int32_t longitude = atoi( lon );
	  uint32_t lock_time = ( time/1000 );

	  for ( int x = 0; x < 90; x++ ) {
		  buffer[ x ] = '0';
	  }

	  sprintf( buffer, "%08X%08X%04X", lattitude, longitude, 0 );

	  radio_send_multibyte_msg( buffer, 10 );

}

void alarmAndAwayModeStep() {

	LIS3MDLinit();

	uint8_t lat[ 20 ];
	uint8_t lon[ 20 ];


	while( 1 ) {

		if ( ( (mDevice.last_msg_sent + GPS_MSG_TIMEOUT) < HAL_GetTick() ||  (mDevice.last_msg_sent == 0) ) && mDevice.GPS_STATUS == DEVICE_OFF ) {
			L80Enable();
			mDevice.time_since_last_fix = HAL_GetTick();
		}

		if ( (mDevice.time_since_last_fix + GPS_FIX_TIMEOUT < HAL_GetTick() )  && mDevice.GPS_STATUS == DEVICE_ON ) {
			L80Disable();
			radio_send_system_error( 0,1,0,0 );
			mDevice.last_msg_sent = HAL_GetTick();
		}

		if ( checkIfFix( lat, lon ) == FIX  && mDevice.GPS_STATUS == DEVICE_ON ) {
			mDevice.time_since_last_fix = HAL_GetTick() - mDevice.time_since_last_fix;
			sendEncodedGPS(  lat, lon, mDevice.time_since_last_fix  );
			mDevice.time_since_last_fix = HAL_GetTick();
			mDevice.last_msg_sent = HAL_GetTick();
			L80Disable();
		}

		if ( mDevice.last_ble_check + 100000 < HAL_GetTick() ) {
			if ( getBeaconStatus() == BEACON_IN_RANGE  ) {
				L80Disable();
				mDevice.mode = NEAR_BEACON;
				break;
			}
		}

		//setUpTimeoutAlarm( DEFAULT_NEAR_BEACON_CHECK_TIMEOUT );
		//HAL_PWR_EnterSTOPMode( PWR_LOWPOWERREGULATOR_ON, PWR_STOPENTRY_WFI );

	}
	L80Disable();
}

void nearBeaconModeStep() {
	uint8_t near_beacon_timeout = 0;
	LIS3MDLinit();
	//LIS3MDLclearInterrupt();
	while( 1 ) {
		//if ( acc_int > MOVEMENT_THRESHOLD || near_beacon_timeout > NEAR_BEACON_TIMEOUT_CHECK ) {
			near_beacon_timeout = 0;

			if ( getBeaconStatus() == BEACON_OUT_OF_RANGE ) {
				  //blinkLed( LED2, 2000 );
				  break;
			}else{
				 // blinkLed( LED1, 2000 );
			}
		//}
			//LIS3MDLclearInterrupt();
		//setUpTimeoutAlarm( DEFAULT_NEAR_BEACON_CHECK_TIMEOUT, 0 );

		if ( near_beacon_timeout < 255 ) {
			near_beacon_timeout++;
		}

		setUpTimeoutAlarm( DEFAULT_NEAR_BEACON_CHECK_TIMEOUT );
		HAL_PWR_EnterSTOPMode( PWR_LOWPOWERREGULATOR_ON, PWR_STOPENTRY_WFI );
	}

 	mDevice.mode = AWAY_FROM_BEACON;
}

void HAL_UART_RxHalfCpltCallback(UART_HandleTypeDef *huart)
{
	__IO uint8_t Rxbyte;
	if ( huart == &huart1 ) {
		Rxbyte = uart1buffer[0];
		uartRxBuffWrite(&huart1buffer, Rxbyte);

	}else if( huart == &huart2 ){
		Rxbyte = uart2buffer[0];
		uartRxBuffWrite(&huart2buffer, Rxbyte);

	}

}


void HAL_UART_RxCpltCallback(UART_HandleTypeDef *huart)
{
	//HAL_UART_Transmit( &huart2, "WAAAAAAA\r\n", strlen( "WAAAAAAA\r\n" ), 500 );


	//blinkLed( LED1, 2000 );
	__IO uint8_t Rxbyte;
	if ( huart == &huart1 ) {
		Rxbyte = uart1buffer[1];
		uartRxBuffWrite(&huart1buffer, Rxbyte);


	}else if( huart == &huart2 ){
		Rxbyte = uart2buffer[1];
		uartRxBuffWrite(&huart2buffer, Rxbyte);

	}

}
/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  *
  * @retval None
  */
int main(void)
{
  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */

  /* MCU Configuration----------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
   HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  //HAL_Delay( 10000 );

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();



  MX_DMA_Init();
  MX_I2C1_Init();
  //MX_USART1_UART_Init();
  //MX_USART2_UART_Init();
  MX_RTC_Init();

  /* Initialize interrupts */
  MX_NVIC_Init();
  /* USER CODE BEGIN 2 */
  ALARM_STATUS = NO_ALARM;
  blinkLed( LED1, 4000 );


  uint8_t mode = 255;
  mDevice.last_ble_check = 0;
  mDevice.last_msg_sent = 0;
  mDevice.time_since_last_fix = 0;
  mDevice.GPS_STATUS = DEVICE_OFF;
  mDevice.RADIO_STATUS = DEVICE_OFF;
  mDevice.BLE_STATUS = DEVICE_OFF;

  if ( readModeFromFlash( &mode ) == FLASH_ERROR ) {
 	  //factory test option
 	  mDevice.mode = STORAGE;
 	  writeDeviceStateToFlash( mDevice.mode );
   }else{
 	  mDevice.mode = mode;
 	  //writeDeviceStateToFlash( mDevice.mode );
   }
	HAL_GPIO_WritePin( V_BLE_EN_GPIO_Port, V_BLE_EN_Pin, GPIO_PIN_RESET );

    radio_init();

    radio_send_system_msg( 1, 0, 0 );
  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
    mDevice.mode = 1;
  while (1)
  {

  /* USER CODE END WHILE */

  /* USER CODE BEGIN 3 */
	  if ( mDevice.mode == 0 ) {
		  storageModeStep();
		  //mDevice.mode = 1;
	  }else if ( mDevice.mode == 1 ) {
		  nearBeaconModeStep();
	  }else if( mDevice.mode == 2 ) {
		  alarmAndAwayModeStep();
	  }else if( mDevice.mode == 3 ) {
		  alarmAndAwayModeStep();
	  }

  }
  /* USER CODE END 3 */

}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{

  RCC_OscInitTypeDef RCC_OscInitStruct;
  RCC_ClkInitTypeDef RCC_ClkInitStruct;
  RCC_PeriphCLKInitTypeDef PeriphClkInit;

    /**Configure the main internal regulator output voltage 
    */
  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);

    /**Initializes the CPU, AHB and APB busses clocks 
    */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_LSI|RCC_OSCILLATORTYPE_MSI;
  RCC_OscInitStruct.LSIState = RCC_LSI_ON;
  RCC_OscInitStruct.MSIState = RCC_MSI_ON;
  RCC_OscInitStruct.MSICalibrationValue = 0;
  RCC_OscInitStruct.MSIClockRange = RCC_MSIRANGE_6;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_NONE;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

    /**Initializes the CPU, AHB and APB busses clocks 
    */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_MSI;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_0) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

  PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_USART1|RCC_PERIPHCLK_USART2
                              |RCC_PERIPHCLK_I2C1|RCC_PERIPHCLK_RTC;
  PeriphClkInit.Usart1ClockSelection = RCC_USART1CLKSOURCE_SYSCLK;
  PeriphClkInit.Usart2ClockSelection = RCC_USART2CLKSOURCE_SYSCLK;
  PeriphClkInit.I2c1ClockSelection = RCC_I2C1CLKSOURCE_SYSCLK;
  PeriphClkInit.RTCClockSelection = RCC_RTCCLKSOURCE_LSI;
  if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInit) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

    /**Configure the Systick interrupt time 
    */
  HAL_SYSTICK_Config(HAL_RCC_GetHCLKFreq()/1000);

    /**Configure the Systick 
    */
  HAL_SYSTICK_CLKSourceConfig(SYSTICK_CLKSOURCE_HCLK);

  /* SysTick_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(SysTick_IRQn, 0, 0);
}

/**
  * @brief NVIC Configuration.
  * @retval None
  */
static void MX_NVIC_Init(void)
{
  /* DMA1_Channel4_5_6_7_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(DMA1_Channel4_5_6_7_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(DMA1_Channel4_5_6_7_IRQn);
  /* DMA1_Channel2_3_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(DMA1_Channel2_3_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(DMA1_Channel2_3_IRQn);
  /* RTC_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(RTC_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(RTC_IRQn);
  /* EXTI0_1_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(EXTI0_1_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(EXTI0_1_IRQn);
  /* EXTI4_15_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(EXTI4_15_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(EXTI4_15_IRQn);
}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @param  file: The file name as string.
  * @param  line: The line in file as a number.
  * @retval None
  */
void _Error_Handler(char *file, int line)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  while(1)
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{ 
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/**
  * @}
  */

/**
  * @}
  */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
