/**
  ******************************************************************************
  * @file           : main.h
  * @brief          : Header for main.c file.
  *                   This file contains the common defines of the application.
  ******************************************************************************
  ** This notice applies to any and all portions of this file
  * that are not between comment pairs USER CODE BEGIN and
  * USER CODE END. Other portions of this file, whether 
  * inserted by the user or by software development tools
  * are owned by their respective copyright owners.
  *
  * COPYRIGHT(c) 2018 STMicroelectronics
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __MAIN_H__
#define __MAIN_H__

/* Includes ------------------------------------------------------------------*/
#include "stm32l0xx_hal.h"

/* USER CODE BEGIN Includes */
#include "stdint.h"

/* USER CODE END Includes */

/* Private define ------------------------------------------------------------*/

#define LED_G_Pin GPIO_PIN_14
#define LED_G_GPIO_Port GPIOC
#define LED_R_Pin GPIO_PIN_15
#define LED_R_GPIO_Port GPIOC
#define RECT_Pin GPIO_PIN_0
#define RECT_GPIO_Port GPIOA
#define RECT_EXTI_IRQn EXTI0_1_IRQn
#define V_RADIO_EN_Pin GPIO_PIN_1
#define V_RADIO_EN_GPIO_Port GPIOA
#define TX_RADIO_Pin GPIO_PIN_2
#define TX_RADIO_GPIO_Port GPIOA
#define RX_RADIO_Pin GPIO_PIN_3
#define RX_RADIO_GPIO_Port GPIOA


#define VSEL_Pin GPIO_PIN_0
#define VSEL_GPIO_Port GPIOB
#define FIX_Pin GPIO_PIN_6
#define FIX_GPIO_Port GPIOA
#define V_BLE_EN_Pin GPIO_PIN_4
#define V_BLE_EN_GPIO_Port GPIOA
#define CHG_Pin GPIO_PIN_1
#define CHG_GPIO_Port GPIOB
#define V_GPS_EN_Pin GPIO_PIN_3
#define V_GPS_EN_GPIO_Port GPIOB
#define TX_GPS_Pin GPIO_PIN_10
#define TX_GPS_GPIO_Port GPIOA
#define RX_GPS_Pin GPIO_PIN_9
#define RX_GPS_GPIO_Port GPIOA
#define V_BKP_Pin GPIO_PIN_11
#define V_BKP_GPIO_Port GPIOA
#define V_ACC_Pin GPIO_PIN_5
#define V_ACC_GPIO_Port GPIOB
#define ACC_INT_Pin GPIO_PIN_6
#define ACC_INT_GPIO_Port GPIOB
#define ACC_INT_EXTI_IRQn EXTI4_15_IRQn

/* ########################## Assert Selection ############################## */
/**
  * @brief Uncomment the line below to expanse the "assert_param" macro in the 
  *        HAL drivers code
  */
/* #define USE_FULL_ASSERT    1U */

/* USER CODE BEGIN Private defines */
#define MOVEMENT_THRESHOLD 50
#define NEAR_BEACON_TIMEOUT_CHECK 8

#define TX_BLE_Pin GPIO_PIN_14
#define TX_BLE_GPIO_Port GPIOA

#define RX_BLE_Pin GPIO_PIN_15
#define RX_BLE_GPIO_Port GPIOA


typedef enum ALARMStatus {
	NO_ALARM, ALARM
}ALARMStatus;
volatile ALARMStatus ALARM_STATUS;

//#define SIGFOX
#define MURATA

#define DEVICE_ON 1
#define DEVICE_OFF 0


#define GPGGA  "$GNGGA,******.***,****.****,*,*****.****,*,1"
#define GPGGA1  "$GNGGA,******.***,****.****,*,*****.****,*,2"

#define GPS_MSG_TIMEOUT 300000
typedef enum PetTrackerModes{
	STORAGE = 0, NEAR_BEACON = 1, AWAY_FROM_BEACON = 2, ALARM_AND_AWAY = 3

} PetTrackerModes;
typedef struct MainDevice {
	PetTrackerModes mode;
	uint64_t last_ble_check;
	uint64_t last_msg_sent;
	uint64_t time_since_last_fix;
	uint8_t BLE_STATUS;
	uint8_t GPS_STATUS;
	uint8_t RADIO_STATUS;


}MainDevice;

typedef enum Errors {
	DMA_ERROR = 0, UART_ERROR = 1, ACC_ERROR = 2, MEMORY_ERROR = 3, RADIO_ERROR = 4
}Errors;

#define DEFAULT_NEAR_BEACON_CHECK_TIMEOUT 300
#define GPS_FIX_TIMEOUT 200000
/* USER CODE END Private defines */

#ifdef __cplusplus
 extern "C" {
#endif
void _Error_Handler(char *, int);

#define Error_Handler() _Error_Handler(__FILE__, __LINE__)
#ifdef __cplusplus
}
#endif

#endif /* __MAIN_H__ */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
