/*
 * radio.c
 *
 *  Created on: Aug 22, 2017
 *      Author: fiala
 */
#include "radio.h"

void radio_send_KA( uint8_t temp  ) {

	uint8_t msg = 0b00000000;
	msg |= (temp & 0b00111111 );

	char hex[2];
	dataToHex(hex, msg);
	send_msg( hex, POWER_20 );

}

void radio_send_system_error( uint8_t acc_err, uint8_t gps_failed_fix, uint8_t gps_global_fail, uint8_t ble_fail  ) {

	uint8_t msg = 0b00100000;
	msg |= acc_err << 3;
	msg |= gps_failed_fix << 2;
	msg |= gps_global_fail << 1;
	msg |= ble_fail;

	char hex[2];
	dataToHex(hex, msg);
	send_msg( hex, POWER_20 );

}

void radio_send_system_msg( uint8_t system_start, uint8_t first_charge, uint8_t low_battery  ) {

	uint8_t msg = 0b00000000;
	msg |= low_battery << 2;
	msg |= system_start << 1;
	msg |= first_charge;

	char hex[2];
	dataToHex(hex, msg);
	send_msg( hex, POWER_20 );

}

void send_msg( char * hex, uint8_t power ) {

	uint8_t state = CAN_SEND;
	if ( state == CANT_SEND ) {
		return;
	}else if ( state == LAST_SEND ) {
		radio_on();
		HAL_Delay( 25 );
		radio_set_power( POWER_20 );
		HAL_Delay( 100 );
		radio_send_msg( "64" );
		HAL_Delay( 25 );
		radio_off();
		return;
	}

	radio_on();
	/*
	HAL_UART_Transmit( &huart2, "\r", 1, 20 );
	HAL_Delay( 25 );
	*/
	HAL_Delay( 4000 );
	radio_set_power( power );
	HAL_Delay( 25 );
	radio_send_msg( hex );
	HAL_Delay( 5000 );
	radio_off();

}


void radio_on() {
	uart_type = UART_RADIO;
	#ifdef MURATA
		lora_on();
	#endif

	#ifdef SIGFOX
		sigfox_on();
	#endif
}

void radio_off() {
	#ifdef MURATA
		lora_off();
	#endif

	#ifdef SIGFOX
		sigfox_off();
	#endif
}

uint8_t radio_recv_msg() {
#ifdef MURATA
	return lora_recv_msg();
#endif

#ifdef SIGFOX
	return sigfox_recv_msg();
#endif
}

uint8_t radio_frame_counter_inc() {

#ifdef MURATA
	lora_init();
#endif

#ifdef SIGFOX
	sigfox_frame_counter();
#endif


}

void radio_init(void)
{

	#ifdef MURATA
		lora_init();
	#endif

	#ifdef SIGFOX
		sigfox_init();
	#endif
}

void radio_set_power( uint8_t power ) {
	#ifdef MURATA
		if ( power == POWER_20 ) {
			HAL_UART_Transmit( &huart2, "\r", 1, DEFAULT_UART_TIMEOUT );
			HAL_Delay( 25 );
			uartRxBuffInit( &huart2buffer );

			HAL_UART_Transmit( &huart2, "AT+RFPOWER=1,0\r", strlen( "AT+RFPOWER=1,0\r" ), DEFAULT_UART_TIMEOUT );
			HAL_Delay( 500 );
		}else if( power == POWER_17 ) {
			lora_send( "AT+RFPOWER=1,1\r" );	//14dBm
		}else if( power == POWER_14 ) {
			lora_send( "AT+RFPOWER=0,3\r" );	//14dBm
		}
	#endif

	#ifdef SIGFOX
		if ( power == POWER_20 ) {
			sigfox_send( "ATS302=20\r" );
		}else if( power == POWER_17 ) {
			sigfox_send( "ATS302=17\r" );
		}else if( power == POWER_16 ) {
			sigfox_send( "ATS302=16\r" );
		}else if( power == POWER_15 ) {
			sigfox_send( "ATS302=15\r" );
		}else if( power == POWER_14 ) {
			sigfox_send( "ATS302=14\r" );	//14dBm
		}

	#endif
}

void radio_send_msg_multi( char * hex, uint8_t multi ) {

	#ifdef MURATA
		char msg_radio[14] = "\rAT+UTX 1\rxx\r\0";

		memcpy(msg_radio+10, hex, 2);
		lora_send( msg_radio );
		//HAL_UART_Transmit( &huart2, "\r", 1, 20 );
		//HAL_UART_Transmit( &huart2, msg_radio, 13, DEFAULT_UART_TIMEOUT );
		// toto nie je finalne riesenie, treba prepracovat!!!!
	#endif

#ifdef SIGFOX
	char msg_radio[40] = "\rAT$SF=xx\r\0";
	memcpy(msg_radio+7, hex, multi);

	msg_radio[ 7 + multi ] = "\r";
	msg_radio[ 7 + multi + 1 ] = "\0";

	sigfox_send( msg_radio );
	//HAL_UART_Transmit( &huart2, "\r", 1, 20 );
	//HAL_UART_Transmit( &huart2, msg_radio, 13, DEFAULT_UART_TIMEOUT );
		// toto nie je finalne riesenie, treba prepracovat!!!!
#endif


}


void radio_send_msg( char * hex ) {

	#ifdef MURATA
		char msg_radio[14] = "\rAT+UTX 1\rxx\r\0";

		memcpy(msg_radio+10, hex, 2);
		lora_send( msg_radio );
		//HAL_UART_Transmit( &huart2, "\r", 1, 20 );
		//HAL_UART_Transmit( &huart2, msg_radio, 13, DEFAULT_UART_TIMEOUT );
		// toto nie je finalne riesenie, treba prepracovat!!!!
	#endif

#ifdef SIGFOX
	char msg_radio[14] = "\rAT$SF=xx\r\0";
	memcpy(msg_radio+7, hex, 2);
	sigfox_send( msg_radio );
	//HAL_UART_Transmit( &huart2, "\r", 1, 20 );
	//HAL_UART_Transmit( &huart2, msg_radio, 13, DEFAULT_UART_TIMEOUT );
		// toto nie je finalne riesenie, treba prepracovat!!!!
#endif


}

